/* rubba_b.com C conversion by kometbomb in the year of 2007 */

#include <math.h>
#include <windows.h>
#include <stdio.h>

#include "tinyptc/tinyptc.h"

const float sinestep=0.0001917476;
const unsigned short int sinescale=0x50;

#define SINETAB 65536
#define FACES 4

/* tables */

short int sinetab[SINETAB];
unsigned char palette[256*4]; // 256 * RGBA
unsigned char texture[256*256];
unsigned char screen[320*200];

/* leet x86 arithmetic emulation */

static unsigned char carry=0; // the carry flag

/* 8-bit rotate left thru carry */
inline unsigned char rcl(unsigned char reg,unsigned char count) {
	/* the following is p00p but I really don't feel like 
		 fighting with types and signedness...
	*/
	if (count&0x80)
		count=(((count+1)%9)&0xff);
	else
		count=((count)%9)&0xff;
		
	if (!count)	return reg; // X rcl 9 == X
	
	const unsigned char result=(reg<<count) | (carry<<(count-1)) | (reg>>(9-count));
	carry=(reg>>(8-count)) & 1;		
	
	return result&0xff;
}

/* 8-bit add with carry */
inline unsigned char adc(unsigned char reg,unsigned char reg2) {
	const short int result=(unsigned short int)reg+(unsigned short int)carry+(unsigned short int)reg2;
	
	carry=(result>255); 
	
	return result;
}

/* 8-bit signed shift left */
inline unsigned char sar(char reg,unsigned char count) {
	carry=(reg>>(count-1)) & 1;
	return reg>>count;
}

/* 8-bit unsigned shift right */
inline unsigned char shr(unsigned char reg,unsigned char count) {
	carry=(reg>>(count-1)) & 1;
	return reg>>count;
}

/* --------- */

inline void gensine() {
	float st=0;
	register unsigned short int bx=0;
	for (;;) {
		sinetab[bx]=cos(st)*sinescale;
		st+=sinestep;
		if (!--bx) break;
	}
}

inline void pal() {
	register unsigned char cl=0;
	
	for(;;) {
		register unsigned char al=cl;
			
		unsigned char *palptr=&palette[(int)cl*4];
		
		*(palptr++)=4*(sar(al,1)&15); // this is in bgr order
		*(palptr++)=4*sar(al,1);
		*(palptr++)=4*al; // times four because vga palette is 6-bit and we're in 8-bit per color mode
		
		if (!--cl) break;
	}
}



inline void tex() {
	unsigned short int cx=0;
	unsigned char dh=0x3,ah=0,al=0;
	
	for (;;) {
		dh=rcl(dh,cx);
		ah=sar(dh,3);
		al=adc(al,ah);
		al=adc(al,texture[(cx+32)&0xffff]);
		al=shr(al,1);
		texture[cx]=al;
		if (!--cx) break;
	}
}

static short unsigned int rot=0,sofs=0x1234;

inline void mainloop() {
	int bx,ax,bp,di,si;
	int dh,dl;
	int yofs=160,tofs=0;
	int cx;
	register unsigned char col;
	
	
	// clear screen
	
	for (cx=0;cx<320*200;cx++) {
		screen[cx]=0;
	}
	
	// draw rows
	
	bx=rot&(SINETAB-1);
	
	for (dh=200;dh;dh--) {
		for (dl=FACES;dl;dl--) {
			ax=sinetab[bx&(SINETAB-1)]; // first vertex
			bx+=SINETAB/FACES/2;
			bp=sinetab[bx&(SINETAB-1)]; // second vertex
			
			di=ax;
			cx=ax-bp; // line length
			
			if (cx<0) cx=-cx; // negative length?
			
			di+=yofs;
			
			if (ax<bp) {
				col=cx;
				bp=ax=((ax&0xff)|((256/FACES)*256))/cx;
				si=tofs;
				for(;cx;cx--) {
					
					si+=bp;
					
					const unsigned char al=texture[((si&0xff00)+dh)&0xffff];
					screen[di++]=((short unsigned int)al*(short unsigned int)col)/256;
				}
				// drawline
			}
			
			tofs+=(256/FACES)*256;
		}
		
		
		yofs+=320; // next line, which is 320 pixels further
		
		bx+=sinetab[sofs&(SINETAB-1)]; // rotate next row 
		
		sofs+=1;
	}
	
	rot-=3*256/2;
	
}

inline void swapscreen() {
	static int pixels[320*200];	
	int c,*dptr=pixels;
	unsigned char *sptr=screen;
	for (c=0;c<320*200;c++)
		*(dptr++)=((int*)palette)[*(sptr++)];
		
	ptc_update(pixels);

}

inline int moval13h() {
	if (!ptc_open("rubba_b",320,200)) return 1;
	return 0;
}

int main(int argc,char **argv) {
	if (moval13h()) return 1;
	
	gensine();
	pal();
	tex();
	
	for(;;) {
		mainloop();
		swapscreen();
		Sleep(20);
	} 
	
	return 0;	
}
